package be.iminds.ilabt.jfed.experimenter_gui.gts_editor;

import be.iminds.ilabt.jfed.experimenter_gui.gts_editor.ribbon_tabs.GtsChildTopologyEditorRibbonTab;
import be.iminds.ilabt.jfed.experimenter_gui.gts_editor.views.GTSViewsFactory;
import be.iminds.ilabt.jfed.experimenter_gui.tabs.AskOnCloseRequest;
import be.iminds.ilabt.jfed.experimenter_gui.tabs.RibbonEnabled;
import be.iminds.ilabt.jfed.experimenter_gui.tabs.StatusEnabled;
import be.iminds.ilabt.jfed.experimenter_gui.ui.ribbon.RibbonTab;
import be.iminds.ilabt.jfed.gts.model.impl.FXGtsType;
import be.iminds.ilabt.jfed.ui.javafx.GlyphUtils;
import be.iminds.ilabt.jfed.ui.javafx.dialogs.JFDialogs;
import javafx.beans.value.ObservableStringValue;
import javafx.scene.control.ContextMenu;
import javafx.scene.control.MenuItem;
import javafx.scene.control.Tab;
import javafx.scene.control.TextInputDialog;
import org.controlsfx.glyphfont.FontAwesome;

import java.util.ArrayList;
import java.util.List;

import static javafx.beans.binding.Bindings.concat;

/**
 * User: twalcari
 * Date: 5/18/2015
 * Time: 10:10
 */
public class GtsChildTypeEditorTab extends Tab implements RibbonEnabled, StatusEnabled, AskOnCloseRequest {
    private final static List<RibbonTab> ribbonTabs;

    private static GtsChildTopologyEditorRibbonTab topologyEditorRibbonTab;

    private final FXGtsType type;
    private final GTSModelEditor parentEditor;
    private final GtsChildTypeEditor gtsChildTypeEditor;


    static {
        ribbonTabs = new ArrayList<>();

        topologyEditorRibbonTab = new GtsChildTopologyEditorRibbonTab();

        ribbonTabs.add(topologyEditorRibbonTab);
    }

    GtsChildTypeEditorTab(FXGtsType type, GTSModelEditor parentEditor, GTSViewsFactory gtsViewsFactory) {
        this.type = type;
        this.parentEditor = parentEditor;

        textProperty().bind(concat("[ ", parentEditor.getModel().nameProperty(), "] ", type.nameProperty()));

        getStyleClass().add("tab-gts-design");
        setGraphic(GlyphUtils.createGlyph(FontAwesome.Glyph.CLOUD));

        ContextMenu contextMenu = new ContextMenu();
        MenuItem changeNameMenuItem = new MenuItem("Change name", GlyphUtils.createGlyph(FontAwesome.Glyph.PENCIL));
        changeNameMenuItem.setOnAction(actionEvent -> showEditNameDialog());
        contextMenu.getItems().add(changeNameMenuItem);
        setContextMenu(contextMenu);


        this.gtsChildTypeEditor = new GtsChildTypeEditor(type, parentEditor, gtsViewsFactory);
        setContent(gtsChildTypeEditor);
    }

    private void showEditNameDialog() {
        String name = null;

        while (name == null) {
            TextInputDialog dialog = new TextInputDialog(type.getName());
            dialog.initOwner(getTabPane().getScene().getWindow());
            dialog.setTitle("Change GTS Type name");
            dialog.setHeaderText("Change GTS Type name");
            dialog.setContentText("Enter a new name:");
            name = dialog.showAndWait().orElse(null);

            if (name == null) {
                return; //user cancelled
            }
            name = name.trim();

            if (!EditableGtsDsl.verifyName(name)) {
                JFDialogs.create()
                        .message("The given name is not a valid for GTS. Please choose a name using the following characters: [A-Za-z0-9_]")
                        .showError();

                name = null;
            }
        }

        type.setName(name);
    }


    public List<? extends RibbonTab> getRibbonTabs() {
        return ribbonTabs;
    }

    public RibbonTab getInitialTabToActivate() {
        return topologyEditorRibbonTab;
    }

    public ObservableStringValue statusProperty() {
        return gtsChildTypeEditor.statusProperty();
    }

    public GtsChildTypeEditor getGtsChildTypeEditor() {
        return gtsChildTypeEditor;
    }

    @Override
    public boolean onTabCloseRequest() {
        //FIXME Check if child items are open before allowing to close
        return true;
    }
}
